<?php

require_once __DIR__.DS.'polyfill.php';

/**
 * Edit products' info in one place
 */
class BulkProductAttributeInfo extends Module
{
    /**
     * @constructor
     */
    public function __construct() {
        $this->name = 'bulkproductattributeinfo';
        $this->tab = 'administration';
        $this->version = '1.0';
        $this->author = 'Simone Salerno';

        parent::__construct();

        $this->displayName = $this->l('Bulk product attribute info');
        $this->description = $this->l('Edit products\' attributes info in one place');
    }
    
    /**
     * Show form
     * @return string
     */
    public function getContent()
    {
        if (Tools::isSubmit('bulkproductattributeinfo_submit')) {
            $submit = Tools::getValue('bulkproductattributeinfo_data');
            $update = $this->updateProductAttributeInfo(json_decode($submit));
        }
        
        Tools::addCSS(__DIR__.DS.'handsontable.full.css');
        Tools::addJS(__DIR__.DS.'handsontable.full.js');
        
        $combinations = $this->castIntToBool(
            $this->getBulkProductAttributesInfo(),
            ['default_on']
        );
        $this->context->smarty->assign('bulkproductattributesinfo', $combinations);
        
        return (isset($update) ? $update : '') . $this->display(__FILE__, 'bulkproductattributeinfo.tpl');
    }
    
    /**
     * Get products info
     * @return array
     */
    private function getBulkProductAttributesInfo()
    {
        $query = (new DbQuery)
                ->select('pa.id_product_attribute, pa.id_product, pl.name AS product_name')
                ->select('GROUP_CONCAT(CONCAT(agl.name, " ", al.name), " - ") AS combination_name')
                ->select('pa.reference, pa.ean13, pa.upc, pas.minimal_quantity, pa.location')
                ->select('pas.price, pas.wholesale_price, pas.ecotax, pas.unit_price_impact')
                ->select('pas.weight, pas.default_on, pas.available_date')
                ->from('product_attribute_shop', 'pas')
                ->innerJoin('product_attribute', 'pa', 'pa.id_product_attribute = pas.id_product_attribute')
                ->innerJoin('product', 'p', 'p.id_product = pa.id_product')
                ->innerJoin('product_lang', 'pl', 'pl.id_product = p.id_product AND pl.id_shop = pas.id_shop')
                ->innerJoin('product_attribute_combination', 'pac', 'pac.id_product_attribute = pa.id_product_attribute')
                ->innerJoin('attribute', 'a', 'a.id_attribute = pac.id_attribute')
                ->innerJoin('attribute_lang', 'al', 'al.id_attribute = a.id_attribute')
                ->innerJoin('attribute_group', 'ag', 'ag.id_attribute_group = a.id_attribute_group')
                ->innerJoin('attribute_group_lang', 'agl', 'agl.id_attribute_group = ag.id_attribute_group')
                ->where(sprintf(
                        'pl.id_lang = %1$d AND agl.id_lang = %1$s AND al.id_lang = %1$s AND pas.id_shop = %2$d', 
                        (int) $this->context->language->id,
                        (int) $this->context->shop->id
                ))
                ->groupBy('pa.id_product_attribute')
                ->orderBy('product_name, combination_name');
        $rows = Db::getInstance()->executeS($query);
        
        return array_map(function($row) {
            $row['name'] = sprintf('%s :: %s', $row['product_name'], preg_replace('/ - $/', '', $row['combination_name']));
            unset($row['product_name']);
            unset($row['combination_name']);
            return $row;
        }, $rows);
    }
    
    /**
     * Update product info
     * @param array $combinations_info
     * @return string
     */
    private function updateProductAttributeInfo($records)
    {
        $language = $this->context->language;
        $shop = $this->context->shop;
        $response = [];
        
        foreach ((array) $records as $record) {
            $message = $this->displayConfirmation(sprintf("Updated %s", $record->name));
            
            try {
                $combination = new Combination($record->id_product_attribute);
                $this->fill($combination, (array) $record);
                $combination->save();
            }
            catch (Exception $ex) {
                $message = $this->displayError(sprintf("Error on %s: [%s]", $record->name, $ex->getMessage()));
            }
            
            array_push($response, $message);
        }
        
        return implode('', $response);
    }
    
    /**
     * Copy data from submit form to combiantion object
     * @param Combination $combination
     * @param array $data
     */
    private function fill(Combination $combination, array $data)
    {
        foreach ($data as $key => $value) {
            if (property_exists($combination, $key))
                $combination->{$key} = $value;
        }
        
        //format date
        if (isset($data['available_date'])) {
            $time = str_replace('/', '-', $data['available_date']);
            $date = date('Y-m-d H:i:s', strtotime($time));
            $combination->available_date = $date;
        }
    }
    
    /**
     * Cast 0/1 attributes to true/false
     * @param array $records
     * @param array $attributes
     * @return array
     */
    private function castIntToBool($records, $attributes)
    {
        return array_map(function($record) use ($attributes) {
            foreach ($attributes as $attribute) {
                if (array_key_exists($attribute, $record)) {
                    $record[$attribute] = (bool) $record[$attribute];
                }
            }
                
            return $record;
        }, $records);
    }
}